﻿Type=Service
Version=2.52
@EndOfDesignText@
#Region  Service Attributes 
	#StartAtBoot: False
#End Region

'This Service: 
'	-Communication with the PLC through a Socket and AsyncSteamsText
'	-Has a timer providing that PLC-register changes regularly are read in to the PLCRegister objects
'	-Keeps a list NewValueRegisterList that is managed by the service as a queue for sending new values to the PLC
'	-The queue contains a type that contains:
'		*the PLCRegister-Object for the register for the new value
'		*the new value
'		*the Presentation-object that is the source of the new value 
'	-Provide that the new values in the queue are written to the PLC
'   -Provide that the Presentation-object is informed when the value is written to the PLC

Sub Process_Globals
	'These global variables will be declared once when the application starts.
	'These variables can be accessed from all modules.
	Private SocketPLC As Socket
	Private SocketPLCConnectionFailed As Boolean
	Private SocketPLCException As String
	Private SocketPLCConnecting As Boolean
	
	Public TimerReadPLC As Timer
	Private DateTimeRequestSentToPLC As Long	

	Private TimerTimeoutCom As Timer
	Private RemainingTries As Short

	Private NewValueRegisterList As List
	Type StartWriteValueToPLCArguments(PLCReg As PLCRegister, SourcePresentation As Object, newValue As Short)
	Private TimeToReadPLC As Boolean
	

	Private Stream As AsyncSteamsText
	
End Sub

Sub Service_Create

End Sub

Sub Service_Start (StartingIntent As Intent)
End Sub

Sub Service_Destroy
End Sub


Sub CommunicationCommand
	Dim lPort As Int
	Dim lHost As String

	Select Case True
	Case CommunicationStatus = 0 OR CommunicationStatus = 1 'Not connected or Not connected, connecting failed
		SocketPLCConnectionFailed = False
		SocketPLCConnecting  = True
		
		lPort = Main.PortPLC
		lHost = Main.HostPLC
		
		SocketPLC.Initialize("SocketPLC")
		SocketPLC.Connect(lHost , lPort,10000) 

		CallSub(Main, "CommunicationStatusChanged")
	Case CommunicationStatus = 2 'Connecting
	Case CommunicationStatus = -1 'Connected
		SocketPLC.Close
		SocketPLCConnectionFailed = False		
		CallSub(Main, "CommunicationStatusChanged")
	End Select
End Sub

Sub HostPLCChange(newHost As String)
	Main.HostPLC = newHost
	If CommunicationStatus <= 0 Then
		SocketPLC.Close
		SocketPLCConnecting = False
		CallSub(Main, "CommunicationStatusChanged")	
	End If
End Sub

Sub PortPLCChange(newPort As String)
	If IsNumber(newPort) = True Then
	   Main.PortPLC = newPort
	Else
	   Main.PortPLC = -1
	End If	
	If CommunicationStatus <= 0 Then
		SocketPLC.Close
		SocketPLCConnecting = False
		CallSub(Main, "CommunicationStatusChanged")	
	End If
End Sub

Sub CommunicationStatus As Int
	If SocketPLC.Connected Then
		Return -1 'Connected
	Else If SocketPLCConnecting Then
		Return 2 'Connecting
	Else If SocketPLCConnectionFailed Then
		Return 1 'Not connected, connecting failed
	Else
		Return 0 'Not connected
	End If	
End Sub

Sub SocketPLC_Connected (Successful As Boolean)
	SocketPLCConnecting = False
	If Successful Then
		StateManager.SetSetting("Host",Main.HostPLC)
		StateManager.SetSetting("Port",Main.PortPLC)
		StateManager.SaveSettings
		If Stream.IsInitialized Then Stream.Close
		Stream.Initialize(Me,"Stream",SocketPLC.InputStream,SocketPLC.OutputStream )
		TimerReadPLC.Initialize("TimerReadPLC", Main.RefreshInterval)
		TimerReadPLC.Enabled = True
		TimerReadPLC_Tick
	Else
		TimerReadPLC.Enabled = False
		TimerTimeoutCom.Enabled = False
		SocketPLCConnectionFailed = True
		SocketPLCException = LastException.Message
		ToastMessageShow(SocketPLCException,True)
	End If
	CallSub(Main, "CommunicationStatusChanged")	
End Sub

Sub TimerReadPLC_Tick
	TimeToReadPLC = True
	If TimerTimeoutCom.Enabled = False Then
		RemainingTries = Main.NumberOfRetries
		SendRequestToPLC
	End If
End Sub

Sub StartWriteValueToPLC(Args As StartWriteValueToPLCArguments)
	If Args.newValue <> Args.PLCReg.Value Then
		If Not(NewValueRegisterList.IsInitialized) Then NewValueRegisterList.Initialize
		NewValueRegisterList.Add(Args)
		If TimerTimeoutCom.Enabled = False Then
			RemainingTries = Main.NumberOfRetries
			SendRequestToPLC
		End If
	Else
		CallSub(Args.SourcePresentation,"PLCregNewValueReceived")
	End If
End Sub

Sub TimerTimeoutCom_Tick
	If RemainingTries > 0 Then
		RemainingTries = RemainingTries - 1
		SendRequestToPLC
	Else
		'Forsøk oppbruka, forsøker å koble til Soccet på ny
		SocketPLC.Close
		CommunicationCommand
	End If
End Sub


Sub SendRequestToPLC
	'Dim RequestString As String
	If NewValueRegisterList.Size > 0 Then
		Dim SogR = NewValueRegisterList.Get(0) As StartWriteValueToPLCArguments
		If SogR.PLCReg.NewValueSentButNotConfirmed Then
			If TimeToReadPLC Then
				PLC.StartRead(1,48, Stream)
			Else
				PLC.StartRead(SogR.PLCreg.Index,SogR.PLCreg.Index, Stream)		
			End If
		Else
			Dim databuffer(1) As Short
			databuffer(0) = SogR.newValue
			PLC.StartWrite( SogR.PLCreg.Index ,SogR.PLCreg.Index, databuffer, Stream)
		End If
	Else If TimeToReadPLC Then
		PLC.StartRead(1,48, Stream)
	End If

	Dim Now = DateTime.Now As Long
	DateTimeRequestSentToPLC = Now 
	TimerTimeoutCom.Initialize("TimerTimeoutCom",Main.TimeOutCom)
	TimerTimeoutCom.Enabled = True
End Sub


Sub Stream_NewText(Input As String)
	DateTimeRequestSentToPLC = 0
	TimerTimeoutCom.Enabled = False
	PLC.Parse(Input)
	If PLC.ActiveRequestSuccessfull Then
		If NewValueRegisterList.Size > 0 Then
			Dim SogR = NewValueRegisterList.Get(0) As StartWriteValueToPLCArguments
			If SogR.PLCreg.NewValueReceived AND Not(SogR.PLCreg.NewValueSentButNotConfirmed) Then
				SogR.PLCreg.NewValueReceived = False
				CallSub(SogR.SourcePresentation, "PLCregNewValueReceived")
				If SogR.PLCreg.Value = SogR.newValue Then NewValueRegisterList.RemoveAt(0)
			End If
		End If		
	End If
	SendRequestToPLC
End Sub

Sub Stream_Terminated
	TimerReadPLC.Enabled = False
	TimerTimeoutCom.Enabled = False
	If LastException.IsInitialized Then
		ToastMessageShow("Communication with PLC broken" & Chr(10) & Chr(13) & LastException.Message,True)
	Else
		ToastMessageShow("Communication with PLC broken"	,True)
	End If
	CallSub(Main, "CommunicationStatusChanged")	
End Sub
